"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }



var _chunkT5OU4655js = require('./chunk-T5OU4655.js');

// src/index.ts
var _fs = require('fs');
var _unplugin = require('unplugin');
var _utils = require('@antfu/utils');

// src/core/options.ts
var _path = require('path');

var _pluginutils = require('@rollup/pluginutils');
var _haspkg = require('has-pkg'); var _haspkg2 = _interopRequireDefault(_haspkg);
function resolveOptions(options = {}) {
  const imports = flattenImportsMap(options.imports, options.presetOverriding);
  const { dts = _haspkg2.default.call(void 0, "typescript") } = options;
  const resolved = _chunkT5OU4655js.__spreadProps.call(void 0, _chunkT5OU4655js.__spreadValues.call(void 0, {
    sourceMap: false,
    resolvedImports: {},
    presetOverriding: false
  }, options), {
    dts: dts === false ? false : dts === true ? _path.resolve.call(void 0, "auto-imports.d.ts") : _path.resolve.call(void 0, dts),
    imports,
    resolvers: _utils.toArray.call(void 0, options.resolvers),
    idFilter: _pluginutils.createFilter.call(void 0, options.include || [/\.[jt]sx?$/, /\.vue\??/, /\.svelte$/], options.exclude || [/node_modules/, /\.git/])
  });
  return resolved;
}
function flattenImportsMap(map, overriding = false) {
  const flat = {};
  _utils.toArray.call(void 0, map).forEach((definition) => {
    if (typeof definition === "string") {
      if (!_chunkT5OU4655js.presets[definition])
        throw new Error(`[auto-import] preset ${definition} not found`);
      const preset = _chunkT5OU4655js.presets[definition];
      definition = typeof preset === "function" ? preset() : preset;
    }
    for (const mod of Object.keys(definition)) {
      for (const id of definition[mod]) {
        const meta = {
          module: mod
        };
        if (Array.isArray(id)) {
          meta.name = id[1];
          meta.from = id[0];
        } else {
          meta.name = id;
        }
        if (flat[meta.name] && !overriding)
          throw new Error(`[auto-import] identifier ${meta.name} already defined with ${flat[meta.name].module}`);
        flat[meta.name] = meta;
      }
    }
  });
  return flat;
}

// src/core/transform.ts
var _magicstring = require('magic-string'); var _magicstring2 = _interopRequireDefault(_magicstring);
var excludeRE = [
  /\bimport\s*([\w_$]*?),?\s*\{([\s\S]*?)\}\s*from\b/g,
  /\bfunction\s*([\s\S]+?)\s*\(/g,
  /\b(?:const|let|var)\s*([\w\d_$]+?)\b/g
];
var matchRE = /\b(\w+)\b/g;
var importAsRE = /^.*\sas\s+/;
var multilineCommentsRE = /\/\*(.|[\r\n])*?\*\//gm;
var singlelineCommentsRE = /\/\/.*/g;
function stripeComments(code) {
  return code.replace(multilineCommentsRE, "").replace(singlelineCommentsRE, "");
}
function transform(code, id, {
  imports,
  sourceMap,
  resolvers,
  resolvedImports = {}
}) {
  const noComments = stripeComments(code);
  const identifiers = new Set(Array.from(noComments.matchAll(matchRE)).map((i) => i[1]));
  if (!identifiers.size)
    return null;
  for (const regex of excludeRE) {
    Array.from(noComments.matchAll(regex)).flatMap((i) => {
      var _a, _b;
      return [...((_a = i[1]) == null ? void 0 : _a.split(",")) || [], ...((_b = i[2]) == null ? void 0 : _b.split(",")) || []];
    }).map((i) => i.replace(importAsRE, "").trim()).forEach((i) => identifiers.delete(i));
  }
  if (!identifiers.size)
    return null;
  const modules = {};
  Array.from(identifiers).forEach((name) => {
    let info = resolvedImports[name] || imports[name];
    if (!info && (resolvers == null ? void 0 : resolvers.length)) {
      const resolved = firstNonNullResult(resolvers, name);
      if (resolved) {
        if (typeof resolved === "string") {
          info = {
            module: resolved,
            name,
            from: "default"
          };
        } else {
          info = resolved;
        }
        resolvedImports[name] = info;
      }
    }
    if (!info)
      return;
    if (!modules[info.module])
      modules[info.module] = [];
    modules[info.module].push(info);
  });
  if (!Object.keys(modules).length)
    return;
  const importStatements = Object.entries(modules).map(([moduleName, names]) => {
    const imports2 = names.map(({ name, from }) => from ? `${from} as ${name}` : name).join(", ");
    return `import { ${imports2} } from '${moduleName}';`;
  }).join("");
  const s = new (0, _magicstring2.default)(code);
  s.prependLeft(0, importStatements);
  return {
    code: s.toString(),
    map: sourceMap ? s.generateMap() : null
  };
}
function firstNonNullResult(array, name) {
  for (let i = 0; i < array.length; i++) {
    const res = array[i](name);
    if (res)
      return res;
  }
}

// src/core/dts.ts
function generateDeclration(imports, resolvedImports = {}) {
  const body = [
    ...Object.entries(imports),
    ...Object.entries(resolvedImports)
  ].map(([name, info]) => `  const ${name}: typeof import('${info.module}')['${info.from || name}']`).join("\n");
  return `// Generated by 'unplugin-auto-import'
// We suggest you to commit this file into your source control
declare global {
${body}
}
export {}
`;
}

// src/index.ts
var src_default = _unplugin.createUnplugin.call(void 0, (options) => {
  const resolved = resolveOptions(options);
  if (!Object.keys(resolved.imports).length && !resolved.resolvers.length)
    console.warn("[auto-import] plugin installed but no imports has defined, see https://github.com/antfu/unplugin-auto-import#configurations for configurations");
  const generateDeclaration = _utils.throttle.call(void 0, 500, false, () => {
    if (!resolved.dts)
      return;
    _fs.promises.writeFile(resolved.dts, generateDeclration(resolved.imports, resolved.resolvedImports), "utf-8");
  });
  generateDeclaration();
  return {
    name: "unplugin-auto-import",
    enforce: "post",
    transformInclude(id) {
      return resolved.idFilter(id);
    },
    transform(code, id) {
      const res = transform(code, id, resolved);
      if (res && resolved.resolvers.length)
        generateDeclaration();
      return res;
    }
  };
});



exports.src_default = src_default;
